/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Skywriter.
 *
 * The Initial Developer of the Original Code is
 * Mozilla.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Walker (jwalker@mozilla.com)
 *   Kevin Dangoor (kdangoor@mozilla.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

define(function(require, exports, module) {

var basic = require("types/basic");

var typeRegistry = {};

exports.addType = function(name, typeObj) {
    typeRegistry[name] = typeObj;
};

exports.getType = function(name) {
    return typeRegistry[name];
};

var resolveType = function(typeSpec) {
  if (typeof typeSpec === 'string') {
      typeSpec = {name: typeSpec};
  }
  var type = exports.getType(typeSpec.name);
  if (!type) {
      throw new Error("Unknown type: " + typeSpec.name);
  }
  typeSpec.type = type;
  return typeSpec;
};

/**
 * Convert some data from a string to another type as specified by
 * <tt>typeSpec</tt>.
 */
exports.fromString = function(stringVersion, typeSpec) {
    var typeData = resolveType(typeSpec);
    return typeData.type.fromString(stringVersion, typeSpec);
};

/**
 * Convert some data from an original type to a string as specified by
 * <tt>typeSpec</tt>.
 */
exports.toString = function(objectVersion, typeSpec) {
    var typeData = resolveType(typeSpec);
    return typeData.type.toString(objectVersion, typeSpec);
};

/**
 * Convert some data from an original type to a string as specified by
 * <tt>typeSpec</tt>.
 */
exports.isValid = function(originalVersion, typeSpec) {
    var typeData = resolveType(typeSpec);
    return typeData.type.isValid(originalVersion, typeSpec);
};

exports.startup = function(data, reason) {
    // TODO: register this extension point.
    
    // catalog.addExtensionPoint("type", {
    //     "description":
    //         "Commands can accept various arguments that the user enters or that are automatically supplied by the environment. Those arguments have types that define how they are supplied or completed. The pointer points to an object with methods convert(str value) and getDefault(). Both functions have `this` set to the command's `takes` parameter. If getDefault is not defined, the default on the command's `takes` is used, if there is one. The object can have a noInput property that is set to true to reflect that this type is provided directly by the system. getDefault must be defined in that case.",
    //     "indexOn": "name"
    // });
    
    exports.addType("text", basic.text);
    exports.addType("number", basic.number);
    exports.addType("boolean", basic.bool);
    exports.addType("object", basic.object);
    exports.addType("selection", basic.selection);
};

});